"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useHTTPCall;

var _react = require("react");

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var GetResponseJSON = function GetResponseJSON(response) {
  return new Promise(function (resolve, reject) {
    return response.json().then(function (json) {
      return resolve({
        response: response,
        json: json
      });
    }).catch(reject);
  });
};

var makeOptions = function makeOptions(options) {
  if (!options) options = {};
  if (!options.headers) options.headers = {};
  if (!options.headers["Content-Type"]) options.headers["Content-Type"] = "application/json;charset=UTF-8";
  return _objectSpread({
    method: "GET",
    get_instantly: true,
    debug: false
  }, options);
};

function useHTTPCall(url) {
  var opt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var options = makeOptions(opt);

  var _useState = (0, _react.useState)(),
      _useState2 = _slicedToArray(_useState, 2),
      body = _useState2[0],
      setBody = _useState2[1];

  var form = {};

  var _useState3 = (0, _react.useState)(),
      _useState4 = _slicedToArray(_useState3, 2),
      startTime = _useState4[0],
      setStartTime = _useState4[1];

  var _useState5 = (0, _react.useState)({
    data: null,
    pending: false,
    loaded: false,
    error: null,
    loadTime: null,
    elapsed: null,
    url: url,
    options: options
  }),
      _useState6 = _slicedToArray(_useState5, 2),
      response = _useState6[0],
      setResponse = _useState6[1];

  var sendRequest = function sendRequest(form_data) {
    setStartTime(new Date());
    setBody(form_data ? form_data : form);
    setResponse(_objectSpread({}, response, {
      pending: true
    }));
  };

  (0, _react.useEffect)(function () {
    if (!startTime) return;
    var debug = options.debug ? console.debug : function () {
      return null;
    };
    var sendBody;
    if (options.method !== "GET" && !_lodash.default.isEmpty(body)) sendBody = body;
    debug("Sending HTTP body:", options.method, url, sendBody);
    fetch(url, {
      method: options.method,
      headers: options.headers,
      body: sendBody && JSON.stringify(sendBody)
    }).then(GetResponseJSON).then(function (output) {
      var loadTime = new Date();
      var elapsed = loadTime.getTime() - startTime.getTime();
      var error = options.checkErrors ? options.checkErrors(output.response, output.json) : null;
      setResponse(_objectSpread({}, response, {
        loadTime: loadTime,
        elapsed: elapsed,
        data: !error && output.json,
        pending: false,
        loaded: true,
        error: error
      }));
      debug("Got HTTP reply:", options.method, url, elapsed + "ms", output.json);
    }).catch(function (error) {
      setResponse(_objectSpread({}, response, {
        error: error,
        pending: false
      }));
    });
  }, [startTime]);

  var requestOnce = function requestOnce() {
    return !response.startTime && !response.pending && !response.loaded && sendRequest();
  };

  var setField = function setField(key, event) {
    var prop = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "value";
    event.persist();
    form[key] = event.target[prop];
  };

  var setForm = function setForm(value) {
    return form = value;
  };

  return {
    response: response,
    sendRequest: sendRequest,
    requestOnce: requestOnce,
    form: form,
    setForm: setForm,
    setField: setField
  };
}